/************************************************************************************
URG RT-Component
Copyright (c) 2010, Segway-Japan, Ltd.
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

#ifndef __SCIP2_H__
#define __SCIP2_H__

#include <stdio.h>


#define SCIP2_TOPURG_ONEANG   (0.25)   //(360 / 1440)
#define SCIP2_TOPURG_SAMPLES  (1080+1) //(270 / ONEANG) + 1 

FILE* Scip2_Start(char* device);
void Scip2_End(FILE*);

int Scip2_GD(FILE* aPort, int aStart, int aEnd, int aSkip, int* scan = NULL);
int Scip2_MD(FILE* aPort, int aStart, int aEnd, int aSkip, int aSkipScan, int aSends, int* scan = NULL);


void Scip2_TimeSync(FILE* aPort, int count);
double Scip2_TimeEstimate(int urg_time);

#endif
